@@ -28,7 +28,7 @@ from collections import namedtuple
 
 import testflows.settings as settings
 
-from .exceptions import DummyTestException, ResultException, TestIteration, DescriptionError
+from .exceptions import DummyTestException, ResultException, TestIteration, DescriptionError, TestRerunIndividually
 from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED, MANDATORY
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
@@ -36,17 +36,18 @@ from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, At
 from .objects import RepeatTest, ExamplesTable
 from .constants import name_sep, id_sep
 from .io import TestIO, LogWriter
-from .name import join, depth, match, absname
+from .name import join, depth, match, absname, split, isabs
 from .funcs import current, top, previous, main, exception, pause, _set_current_top_previous
 from .init import init
 from .cli.arg.parser import ArgumentParser as ArgumentParserClass
 from .cli.arg.common import epilog as common_epilog
 from .cli.arg.exit import ExitWithError, ExitException
-from .cli.arg.type import key_value as key_value_type, repeat as repeat_type, tags_filter as tags_filter_type
+from .cli.arg.type import key_value as key_value_type, repeat as repeat_type, tags_filter as tags_filter_type, logfile as logfile_type
 from .cli.text import danger, warning
 from .exceptions import exception as get_exception
 from .filters import The, TheTags
 from .utils.sort import human as human_sort
+from .transform.log.pipeline import ResultsLogPipeline
 
 try:
     import testflows.database as database_module
@@ -275,7 +276,7 @@ class TestBase(object):
 
         self.io.output.test_message()
 
-        if self.flags & PAUSE_BEFORE:
+        if self.flags & PAUSE_BEFORE and not self.flags & SKIP:
             pause()
 
         self.caller_test = current()
@@ -313,7 +314,7 @@ class TestBase(object):
                     self.result = self.result(OK(test=self.name))
         finally:
             try:
-                if self.type >= TestType.Test:
+                if self.type >= TestType.Iteration:
                     if self.context._cleanups:
                         try:
                             with Finally("I clean up"):
@@ -334,7 +335,7 @@ class TestBase(object):
                 else:
                     self.io.close()
 
-            if self.flags & PAUSE_AFTER:
+            if self.flags & PAUSE_AFTER and not self.flags & SKIP:
                 pause()
 
         return True
@@ -392,6 +393,297 @@ type
 
 """ + common_epilog()
 
+
+def cli_argparser(kwargs, argparser=None):
+    """Command line argument parser.
+
+    :argparser: test specific argument parser
+    :return: argument parser
+    """
+    description = kwargs.get("description")
+    if description is not None:
+        description = str(description)
+
+    main_parser = ArgumentParserClass(
+        prog=sys.argv[0],
+        description=description,
+        description_prog="Test - Framework",
+        epilog=epilog
+    )
+
+    if argparser:
+        argparser(main_parser.add_argument_group('test arguments'))
+
+    parser = main_parser.add_argument_group("common arguments")
+
+    parser.add_argument("--name", dest="_name", metavar="name",
+                        help="test run name", type=str, required=False)
+    parser.add_argument("--tag", dest="_tags", metavar="value", nargs="+",
+                        help="test run tags", type=str, required=False)
+    parser.add_argument("--attr", dest="_attrs", metavar="name=value", nargs="+",
+                        help="test run attributes", type=key_value_type, required=False)
+    parser.add_argument("--only", dest="_only", metavar="pattern", nargs="+",
+                        help="run only selected tests", type=str, required=False)
+    parser.add_argument("--skip", dest="_skip", metavar="pattern",
+                        help="skip selected tests", type=str, nargs="+", required=False)
+    parser.add_argument("--start", dest="_start", metavar="pattern", nargs=1,
+                        help="start at the selected test", type=str, required=False)
+    parser.add_argument("--end", dest="_end", metavar="pattern", nargs=1,
+                        help="end at the selected test", type=str, required=False)
+    parser.add_argument("--only-tags", dest="_only_tags",
+                        help="run only tests with selected tags",
+                        type=tags_filter_type, metavar="type:tag,...", nargs="+", required=False)
+    parser.add_argument("--skip-tags", dest="_skip_tags",
+                        help="skip tests with selected tags",
+                        type=tags_filter_type, metavar="type:tag,...", nargs="+", required=False)
+    parser.add_argument("--pause-before", dest="_pause_before", metavar="pattern", nargs="+",
+                        help="pause before executing selected tests", type=str, required=False)
+    parser.add_argument("--pause-after", dest="_pause_after", metavar="pattern", nargs="+",
+                        help="pause after executing selected tests", type=str, required=False)
+    parser.add_argument("--random", dest="_random_order", action="store_true",
+                        help="randomize order of auto loaded tests", required=False)
+    parser.add_argument("--debug", dest="_debug", action="store_true",
+                        help="enable debugging mode", default=False)
+    parser.add_argument("--no-colors", dest="_no_colors", action="store_true",
+                        help="disable terminal color highlighting", default=False)
+    parser.add_argument("--id", metavar="id", dest="_id", type=str, help="custom test id")
+    parser.add_argument("-o", "--output", dest="_output", metavar="format", type=str,
+                        choices=["classic", "slick", "nice", "quiet", "short", "dots", "raw"], default="nice",
+                        help="""stdout output format, choices are: ['classic', 'slick','nice','short','dots','quiet','raw'],
+            default: 'nice'""")
+    parser.add_argument("-l", "--log", dest="_log", metavar="file", type=str,
+                        help="path to the log file where test output will be stored, default: uses temporary log file")
+    parser.add_argument("--show-skipped", dest="_show_skipped", action="store_true",
+                        help="show skipped tests, default: False", default=False)
+    parser.add_argument("--repeat", dest="_repeat",
+                        help=("number of times to repeat a test until it either fails or passes.\n"
+                              "Where `pattern` is a test name pattern, "
+                              "`number` is a number times to repeat the test, "
+                              "`until` is either {'pass', 'fail', 'complete'} (default: 'fail')"),
+                        type=repeat_type, metavar="pattern,number[,until]]", nargs="+", required=False)
+    parser.add_argument("-r", "--reference", dest="_reference", metavar="log", type=logfile_type("r", encoding="utf-8"),
+                        help="reference log file")
+
+    choices = ["fails", "passes", "xouts", "ok", "fail", "error", "null",
+               "xok", "xfail", "xerror", "xnull", "skip"]
+    parser.add_argument("--rerun", dest="_rerun", metavar="result", type=str,
+                        choices=choices, nargs="+",
+                        help=("rerun tests in the --reference log file.\n"
+                              f"Where `result` is either {choices}"))
+    parser.add_argument("--individually", dest="_individually", action="store_true", default=False,
+                        help="if --rerun is specified then rerun tests in the --reference log file individually.")
+
+    if database_module:
+        database_module.argparser(parser)
+
+    return main_parser
+
+def parse_cli_args(kwargs, parser):
+    """Parse command line arguments.
+
+    :parser: argument parser
+    :return: parsed known arguments
+    """
+    debug_processed = False
+
+    try:
+        args, unknown = parser.parse_known_args()
+        args = vars(args)
+
+        if args.get("_name"):
+            kwargs["name"] = args.pop("_name")
+
+        if args.get("_debug"):
+            settings.debug = True
+            args.pop("_debug")
+
+        debug_processed = True
+
+        if args.get("_no_colors"):
+            settings.no_colors = True
+            args.pop("_no_colors")
+
+        if unknown:
+            raise ExitWithError(f"unknown argument {unknown}")
+
+        if args.get("_id"):
+            settings.test_id = args.get("_id")
+            args.pop("_id")
+
+        if args.get("_log"):
+            logfile = os.path.abspath(args.get("_log"))
+            settings.write_logfile = logfile
+            args.pop("_log")
+        else:
+            settings.write_logfile = os.path.join(tempfile.gettempdir(), f"testflows.{os.getpid()}.log")
+
+        settings.read_logfile = settings.write_logfile
+        if os.path.exists(settings.write_logfile):
+            os.remove(settings.write_logfile)
+
+        settings.output_format = args.pop("_output")
+
+        if args.get("_database"):
+            settings.database = args.pop("_database")
+
+        if args.get("_show_skipped"):
+            settings.show_skipped = True
+            args.pop("_show_skipped")
+
+        if args.get("_random_order"):
+            settings.random_order = True
+            args.pop("_random_order")
+
+        if args.get("_pause_before"):
+            xflags = kwargs.get("xflags", {})
+            for pattern in args.get("_pause_before"):
+                xflags[pattern] = xflags.get(pattern, [0, 0])
+                xflags[pattern][0] |= PAUSE_BEFORE
+            kwargs["xflags"] = xflags
+            args.pop("_pause_before")
+
+        if args.get("_pause_after"):
+            xflags = kwargs.get("xflags", {})
+            for pattern in args.get("_pause_after"):
+                xflags[pattern] = xflags.get(pattern, [0, 0])
+                xflags[pattern][0] |= PAUSE_AFTER
+            kwargs["xflags"] = xflags
+            args.pop("_pause_after")
+
+        if args.get("_only"):
+            only = []
+            for pattern in args.pop("_only"):
+                only.append(The(pattern))
+            kwargs["only"] = only
+
+        if args.get("_skip"):
+            skip = []
+            for pattern in args.pop("_skip"):
+                skip.append(The(pattern))
+            kwargs["skip"] = skip
+
+        if args.get("_start"):
+            pattern = args.pop("_start")[0]
+            kwargs["start"] = The(pattern)
+
+        if args.get("_end"):
+            pattern = args.pop("_end")[0]
+            kwargs["end"] = The(pattern)
+
+        if args.get("_only_tags"):
+            _only_tags = {}
+            for item in args.pop("_only_tags"):
+                _only_tags.update(item)
+            kwargs["only_tags"] = TheTags(**_only_tags)
+
+        if args.get("_skip_tags"):
+            _skip_tags = {}
+            for item in args.pop("_skip_tags"):
+                _skip_tags.update(item)
+            kwargs["skip_tags"] = TheTags(**_skip_tags)
+
+        if args.get("_tags"):
+            kwargs["tags"] = {value for value in args.pop("_tags")}
+
+        if args.get("_attrs"):
+            kwargs["attributes"] = [Attribute(item.key, item.value) for item in args.pop("_attrs")]
+
+        if args.get("_repeat"):
+            repeat = []
+            for item in args.pop("_repeat"):
+                repeat.append(item)
+            kwargs["repeat"] = repeat
+
+        if args.get("_rerun"):
+            rerun_individually = args.pop("_individually", False)
+            rerun = args.pop("_rerun")
+
+            if not args.get("_reference"):
+                raise ExitWithError(f"--reference argument must be specified")
+
+            results = {}
+            ResultsLogPipeline(args.pop("_reference"), results, steps=False).run()
+
+            if kwargs.get("only") is None:
+                kwargs["only"] = []
+
+            if rerun_individually is True:
+                kwargs["rerun_individually"] = []
+
+            RerunTest = namedtuple("RerunTest", "name type tags")
+            rerun_tests = []
+            result_types = []
+
+            for r in rerun:
+                if r == "xouts":
+                    result_types += ["XOK", "XFail", "XError", "XNull"]
+                elif r == "passes":
+                    result_types += ["OK"]
+                elif r == "fails":
+                    result_types += ["Fail", "Error", "Null"]
+                elif r == "ok":
+                    result_types += ["OK"]
+                elif r == "fail":
+                    result_types += ["Fail"]
+                elif r == "error":
+                    result_types += ["Error"]
+                elif r == "null":
+                    result_types += ["Null"]
+                elif r == "xfail":
+                    result_types += ["XFail"]
+                elif r == "xerror":
+                    result_types += ["XError"]
+                elif r == "xnull":
+                    result_types += ["XNull"]
+                elif r == "xok":
+                    result_types += ["XOK"]
+                elif r == "skip":
+                    result_types += ["Skip"]
+
+            for test in results["tests"].values():
+                result = test["result"]
+                test_type = getattr(TestType, result["test_type"])
+                test_name = result["result_test"]
+                test_tags = {tag["tag_value"] for tag in test["test"]["tags"]}
+
+                if test_type >= TestType.Test:
+                    if result["result_type"] in result_types:
+                        found = False
+                        for rerun_test in rerun_tests:
+                            if rerun_test.name.startswith(test_name):
+                                found = True
+                                break
+                        if not found:
+                            rerun_tests.append(RerunTest(test_name, test_type, test_tags))
+
+                rerun_tests.sort()
+                length = len(rerun_tests)
+
+                for i, test in enumerate(rerun_tests):
+                    if i+1 < length and rerun_tests[i+1].name.startswith(test.name):
+                        rerun_tests.remove(test)
+                        i -= 1
+
+            for rerun_test in rerun_tests:
+                if rerun_individually is True:
+                    name_parts = rerun_test.name.split(name_sep)
+                    kwargs["rerun_individually"].append(RerunTest(
+                        The(join(name_sep, name_parts[1], ":", *name_parts[2:], "*")),
+                        rerun_test.type, rerun_test.tags))
+                else:
+                    kwargs["only"].append(The(join(rerun_test.name, "*")))
+
+    except (ExitException, KeyboardInterrupt, Exception) as exc:
+        if not debug_processed or settings.debug:
+            sys.stderr.write(warning(get_exception(), eol='\n'))
+        sys.stderr.write(danger("error: " + str(exc).strip()))
+        if isinstance(exc, ExitException):
+            sys.exit(exc.exitcode)
+        else:
+            sys.exit(1)
+
+    return args
+
 class TestDefinition(object):
     """Test definition.
 
@@ -455,205 +747,11 @@ class TestDefinition(object):
         self.kwargs = kwargs
         self.tags = None
         self.repeat = None
+        self.rerun_individually = None
         self.repeatable_func = None
         self._with_block_frame = None
         return self
 
-    def _argparser(self, argparser=None):
-        """Command line argument parser.
-
-        :argparser: test specific argument parser
-        :return: argument parser
-        """
-        description = self.kwargs.get("description")
-        if description is not None:
-            description = str(description)
-
-        main_parser = ArgumentParserClass(
-            prog=sys.argv[0],
-            description=description,
-            description_prog="Test - Framework",
-            epilog = epilog
-        )
-
-        if argparser:
-            argparser(main_parser.add_argument_group('test arguments'))
-
-        parser = main_parser.add_argument_group("common arguments")
-
-        parser.add_argument("--name", dest="_name", metavar="name",
-                            help="test run name", type=str, required=False)
-        parser.add_argument("--tag", dest="_tags", metavar="value", nargs="+",
-                            help="test run tags", type=str, required=False)
-        parser.add_argument("--attr", dest="_attrs", metavar="name=value", nargs="+",
-                            help="test run attributes", type=key_value_type, required=False)
-        parser.add_argument("--only", dest="_only", metavar="pattern", nargs="+",
-                            help="run only selected tests", type=str, required=False)
-        parser.add_argument("--skip", dest="_skip", metavar="pattern",
-                            help="skip selected tests", type=str, nargs="+", required=False)
-        parser.add_argument("--start", dest="_start", metavar="pattern", nargs=1,
-                            help="start at the selected test", type=str, required=False)
-        parser.add_argument("--end", dest="_end", metavar="pattern", nargs=1,
-                            help="end at the selected test", type=str, required=False)
-        parser.add_argument("--only-tags", dest="_only_tags",
-                            help="run only tests with selected tags",
-                            type=tags_filter_type, metavar="type:tag,...", nargs="+", required=False)
-        parser.add_argument("--skip-tags", dest="_skip_tags",
-                            help="skip tests with selected tags",
-                            type=tags_filter_type, metavar="type:tag,...", nargs="+", required=False)
-        parser.add_argument("--pause-before", dest="_pause_before", metavar="pattern", nargs="+",
-                            help="pause before executing selected tests", type=str, required=False)
-        parser.add_argument("--pause-after", dest="_pause_after", metavar="pattern", nargs="+",
-                            help="pause after executing selected tests", type=str, required=False)
-        parser.add_argument("--random", dest="_random_order", action="store_true",
-                            help="randomize order of auto loaded tests", required=False)
-        parser.add_argument("--debug", dest="_debug", action="store_true",
-                            help="enable debugging mode", default=False)
-        parser.add_argument("--no-colors", dest="_no_colors", action="store_true",
-                            help="disable terminal color highlighting", default=False)
-        parser.add_argument("--id", metavar="id", dest="_id", type=str, help="custom test id")
-        parser.add_argument("-o", "--output", dest="_output", metavar="format", type=str,
-                            choices=["classic", "slick", "nice", "quiet", "short", "dots", "raw"], default="nice",
-                            help="""stdout output format, choices are: ['classic', 'slick','nice','short','dots','quiet','raw'],
-                default: 'nice'""")
-        parser.add_argument("-l", "--log", dest="_log", metavar="file", type=str,
-                            help="path to the log file where test output will be stored, default: uses temporary log file")
-        parser.add_argument("--show-skipped", dest="_show_skipped", action="store_true",
-                            help="show skipped tests, default: False", default=False)
-        parser.add_argument("--repeat", dest="_repeat",
-                            help=("number of times to repeat a test until it either fails or passes.\n"
-                                  "Where `pattern` is a test name pattern, "
-                                  "`number` is a number times to repeat the test, "
-                                  "`until` is either {'pass', 'fail', 'complete'} (default: 'fail')"),
-                            type=repeat_type, metavar="pattern,number[,until]]", nargs="+", required=False)
-        if database_module:
-            database_module.argparser(parser)
-
-        return main_parser
-
-    def _parse_cli_args(self, parser):
-        """Parse command line arguments.
-
-        :parser: argument parser
-        :return: parsed known arguments
-        """
-        kwargs = self.kwargs
-        try:
-            args, unknown = parser.parse_known_args()
-            args = vars(args)
-
-            if args.get("_name"):
-                kwargs["name"] = args.pop("_name")
-
-            if args.get("_debug"):
-                settings.debug = True
-                args.pop("_debug")
-
-            if args.get("_no_colors"):
-                settings.no_colors = True
-                args.pop("_no_colors")
-
-            if unknown:
-                raise ExitWithError(f"unknown argument {unknown}")
-
-            if args.get("_id"):
-                settings.test_id = args.get("_id")
-                args.pop("_id")
-
-            if args.get("_log"):
-                logfile = os.path.abspath(args.get("_log"))
-                settings.write_logfile = logfile
-                args.pop("_log")
-            else:
-                settings.write_logfile = os.path.join(tempfile.gettempdir(), f"testflows.{os.getpid()}.log")
-
-            settings.read_logfile = settings.write_logfile
-            if os.path.exists(settings.write_logfile):
-                os.remove(settings.write_logfile)
-
-            settings.output_format = args.pop("_output")
-
-            if args.get("_database"):
-                settings.database = args.pop("_database")
-
-            if args.get("_show_skipped"):
-                settings.show_skipped = True
-                args.pop("_show_skipped")
-
-            if args.get("_random_order"):
-                settings.random_order = True
-                args.pop("_random_order")
-
-            if args.get("_pause_before"):
-                xflags = kwargs.get("xflags", {})
-                for pattern in args.get("_pause_before"):
-                    xflags[pattern] = xflags.get(pattern, [0, 0])
-                    xflags[pattern][0] |= PAUSE_BEFORE
-                kwargs["xflags"] = xflags
-                args.pop("_pause_before")
-
-            if args.get("_pause_after"):
-                xflags = kwargs.get("xflags", {})
-                for pattern in args.get("_pause_after"):
-                    xflags[pattern] = xflags.get(pattern, [0, 0])
-                    xflags[pattern][0] |= PAUSE_AFTER
-                kwargs["xflags"] = xflags
-                args.pop("_pause_after")
-
-            if args.get("_only"):
-                only = []
-                for pattern in args.pop("_only"):
-                    only.append(The(pattern))
-                kwargs["only"] = only
-
-            if args.get("_skip"):
-                skip = []
-                for pattern in args.pop("_skip"):
-                    skip.append(The(pattern))
-                kwargs["skip"] = skip
-
-            if args.get("_start"):
-                pattern = args.pop("_start")[0]
-                kwargs["start"] = The(pattern)
-
-            if args.get("_end"):
-                pattern = args.pop("_end")[0]
-                kwargs["end"] = The(pattern)
-
-            if args.get("_only_tags"):
-                _only_tags = {}
-                for item in args.pop("_only_tags"):
-                    _only_tags.update(item)
-                kwargs["only_tags"] = TheTags(**_only_tags)
-
-            if args.get("_skip_tags"):
-                _skip_tags = {}
-                for item in args.pop("_skip_tags"):
-                    _skip_tags.update(item)
-                kwargs["skip_tags"] = TheTags(**_skip_tags)
-
-            if args.get("_tags"):
-                kwargs["tags"] = {value for value in args.pop("_tags")}
-
-            if args.get("_attrs"):
-                kwargs["attributes"] = [Attribute(item.key, item.value) for item in args.pop("_attrs")]
-
-            if args.get("_repeat"):
-                repeat = []
-                for item in args.pop("_repeat"):
-                    repeat.append(item)
-                kwargs["repeat"] = repeat
-
-        except (ExitException, KeyboardInterrupt, Exception) as exc:
-            sys.stderr.write(warning(get_exception(), eol='\n'))
-            sys.stderr.write(danger("error: " + str(exc).strip()))
-            if isinstance(exc, ExitException):
-                sys.exit(exc.exitcode)
-            else:
-                sys.exit(1)
-
-        return args
-
     def __call__(self, *pargs, **args):
         if pargs:
             raise TypeError(f"only named arguments are allowed but {pargs} positional arguments were passed")
@@ -690,7 +788,7 @@ class TestDefinition(object):
             format_description = kwargs.pop("format_description", True)
 
             if not top():
-                cli_args = self._parse_cli_args(self._argparser(argparser if not isinstance(argparser, ArgumentParser) else argparser.value))
+                cli_args = parse_cli_args(self.kwargs, cli_argparser(self.kwargs, argparser if not isinstance(argparser, ArgumentParser) else argparser.value))
                 kwargs["args"].update({k: v for k,v in cli_args.items() if not k.startswith("_")})
 
             test = kwargs.pop("test", None)
@@ -797,6 +895,26 @@ class TestDefinition(object):
                 kwargs["flags"] &= ~PAUSE_AFTER
 
             self.repeat = kwargs.pop("repeat", None)
+            self.rerun_individually = kwargs.pop("rerun_individually", None)
+
+            if self.rerun_individually:
+                def transform_pattern(pattern):
+                    if isabs(pattern):
+                        parts = pattern.split(name_sep)
+                        return join(name_sep, parts[1], ":", *parts[2:])
+                    return pattern
+
+                # need to fix all anchored patterns
+                kwargs["xfails"] = {transform_pattern(k): v for k, v in
+                    (kwargs.pop("xfails", {}) or {}).items()} or None
+                kwargs["xflags"] = {transform_pattern(k): v for k, v in
+                    (kwargs.pop("xflags", {}) or {}).items()} or None
+                kwargs["only"] = [The(transform_pattern(str(f))) for f in kwargs.get("only") or []] or None
+                kwargs["skip"] = [The(transform_pattern(str(f))) for f in kwargs.get("skip") or []] or None
+                kwargs["start"] = The(transform_pattern(str(kwargs.get("start")))) if kwargs.get("start") else None
+                kwargs["end"] = The(transform_pattern(str(kwargs.get("end")))) if kwargs.get("end") else None
+                for r in self.repeat or []:
+                    r.pattern.set(transform_pattern(str(r.pattern)))
 
             self.test = test(name, tags=self.tags, description=self.description, repeat=self.repeat, **kwargs)
             if getattr(self, "parent_type", None):
@@ -808,6 +926,12 @@ class TestDefinition(object):
                 self.test._run_outline_with_no_arguments = self.no_arguments
                 self.test._run_outline = True
 
+            if self.rerun_individually is not None:
+                self.trace = sys.gettrace()
+                sys.settrace(dummy)
+                sys._getframe(1).f_trace = functools.partial(self.__rerun_individually__, self.rerun_individually, None, None, None)
+                return
+
             if self.repeat is not None:
                 repeat = self._apply_repeat(name, self.repeat)
                 if repeat is not None:
@@ -942,6 +1066,10 @@ class TestDefinition(object):
         sys.settrace(self.trace)
         raise TestIteration(repeat)
 
+    def __rerun_individually__(self, patterns, *args):
+        sys.settrace(self.trace)
+        raise TestRerunIndividually(patterns)
+
     def __nop__(self, exc_type, exc_value, exc_tb, *args):
         sys.settrace(self.trace)
         raise exc_value.with_traceback(exc_tb)
@@ -1027,6 +1155,50 @@ class TestDefinition(object):
                     test__exit__ = self.test._exit(None, None, None)
                 except(KeyboardInterrupt, Exception):
                     raise
+
+        elif isinstance(exception_value, TestRerunIndividually):
+            try:
+                rerun_tests = exception_value.tests
+
+                self.test._enter()
+
+                if self.repeatable_func is None:
+                    raise Error("not repeatable")
+
+                for i, rerun_test in enumerate(rerun_tests):
+                    __kwargs = dict(self.kwargs)
+                    __kwargs.pop("name", None)
+                    __kwargs.pop("parent", None)
+                    __kwargs["flags"] = Flags(__kwargs.get("flags")) | TE
+                    __kwargs["type"] = TestType.Iteration
+                    __args = __kwargs.pop("args", {})
+
+                    rerun_name = rerun_test.name.pattern.rstrip(name_sep + "*")
+
+                    self._apply_start(rerun_name, self.parent, __kwargs)
+                    self._apply_only_tags(rerun_test.type, rerun_test.tags, __kwargs)
+                    self._apply_skip_tags(rerun_test.type, rerun_test.tags, __kwargs)
+                    self._apply_skip(rerun_name, __kwargs)
+                    self._apply_end(rerun_name, self.parent, __kwargs)
+                    self._apply_only(rerun_name, __kwargs)
+
+                    __only = [rerun_test.name] + (__kwargs.pop("only", []) or [])
+
+                    with Module(name=f"{i}", only=__only, tags=self.tags, **__kwargs) as iteration:
+                        if isinstance(self.repeatable_func, TestOutline):
+                            iteration._run_outline_with_no_arguments = self.no_arguments
+                            iteration._run_outline = True
+                        self.repeatable_func(**__args)
+            except:
+                try:
+                    test__exit__ = self.test._exit(*sys.exc_info())
+                except(KeyboardInterrupt, Exception):
+                    raise
+            else:
+                try:
+                    test__exit__ = self.test._exit(None, None, None)
+                except(KeyboardInterrupt, Exception):
+                    raise
         else:
             try:
                 test__exit__ = self.test._exit(exception_type, exception_value, exception_traceback)
